/* ******************************* C-krk ************************************ */

/* ***************************************************************************** */
/* ***************************************************************************** */
/* ********************* Algorithms for Chess KRK ending *********************** */
/* ***************************************************************************** */
/* ***************************************************************************** */

/* ************** Written by Predrag Janicic and Marko Malikovic (2013) ******** */

/* ***************************************************************************** */
/* For convenience, a KRK position is represented as a sequence of 19 bits:
   ----------------------------------------------------------
   |18|17|16|15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0|
   ----------------------------------------------------------
   |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
   ----------------------------------------------------------
   |bW|  nWRy  |  nWRx  |  nBKy  |  nBKx  |  nWKy  |  nWKx  |    
   ----------------------------------------------------------  */
/* ***************************************************************************** */

#include <time.h>
#include <stdio.h>
#include <stdbool.h>


/* Maximal number of positions */
#define MAX_POS (64*64*64) 

/* Outcomes for positons */
#define DRAW      1000
#define UNKNOWN   1002
#define ILEGAL    1003
#define UNDEF     1004

/* Position (coordinates of three pieces) */
typedef struct pos {
    signed char WKx, WKy, WRx, WRy, BKx, BKy;
} Position;

/* Stategy steps */
enum _eStratStep { 
    eImmediateMate,
    eReadyToMate,
    eSqueeze,
    eApproachDiag,
    eApproachNonDiag,
    eKeepRoomDiag,
    eKeepRoomNonDiag,
    eRookHome,
    eRookSafe,
    eStepMax
};

typedef enum _eStratStep eStratStep;


char* SStep[eStepMax];

/* A record in the lookup table */
typedef struct lur {
    int Pos, OptimalPos, OptimalDist, StrategPos, StrategDist; 
    eStratStep step;
} LookUpRecord;

/* Lookup table */
LookUpRecord LookUpTable[262144];

bool _B_INITIALIZATION_DONE;
bool _B_SETTING_STEPS_DONE;

// Declarations of functions

// Miscellanous 
void Print(Position p);
int Cartesian2Pos(Position p);
void Pos2Cartesian(int i,Position *p);

int MinusPA (int x, int y);
int AbsDiff(int x, int y);
int Distance (int x1,int y1,int x2,int y2);
int ManhattanDistance(int x1,int y1,int x2,int y2);
Position Symmetric(int i, Position p);

// Chess rules 
bool Dim(Position p);
bool RookCaptured(Position p);
bool NotKingNextKing (Position p);
bool NotOnSameSquare (Position p);
bool BlackKingAttacked (Position p);

// Legal moves definitions
bool LegalPosWhiteToMove(Position p) ;
bool LegalPosBlackToMove(Position p) ;
bool WhiteKingMoves(Position p1,Position p2);
bool WhiteRookMoves(Position p1,Position p2);
bool LegalWhiteMove(Position p1, Position p2) ;
bool LegalBlackMove(Position p1, Position p2) ;
bool NextLegalWhiteMove(unsigned i,Position p1,Position *p2) ;
bool NextLegalBlackMove(unsigned i,Position p1,Position *p2) ;

// Mate and Stalemate
bool StalemateOneCase(Position p);
bool Stalemate(Position p);

// Definitions for Strategy
int Room (Position p);
int RoomM (Position p);
bool NewRoomSmaller (Position p1,Position p2);
bool NotBlackKingNextRook (Position p) ;
bool WhiteKingNextRook (Position p) ;
int DistanceWhiteKingToCS (Position p);
int DistanceWhiteKingToRook (Position p);
int DistanceBlackKingToRook (Position p);
int DistanceWhiteKingToBlackKing (Position p);
bool NotWhiteRookExposed (Position p);
bool WhiteRookDivides (Position p);
int CSx(Position p);
int CSy(Position p);
bool ApproachCriticalSquare (Position p1,Position p2);
bool WhiteKingEdge(Position p);
bool LPattern(Position p);
bool WhiteKingAndRookNotDiverging (Position p1,Position p2);

// Strategy (for White)
bool ImmediateMate(Position p1,Position p2);
bool ReadyToMate(Position p1,Position p2);
bool SqueezeCond(Position p1,Position p2);
bool ApproachCondDiag(Position p1,Position p2);
bool ApproachCondNonDiag(Position p1,Position p2);
bool KeepRoomCondDiag(Position p1,Position p2);
bool KeepRoomCondNonDiag(Position p1,Position p2);
bool RookHomeCond(Position p1,Position p2);
bool RookSafeCond(Position p1,Position p2);
Position Strategy(Position p1, eStratStep *s);
Position StrategyM(Position p1, eStratStep *s);


/* ******************************************************************** */
/* Miscellanous                                                         */ 
/* ******************************************************************** */ 

/* -------------------------------------------------------------------------------- */

/* Compressing and uncompressing procedures */

int Cartesian2Pos(Position p)  {
   int i=0;
   i |= p.WKx;
   i <<= 3;
   i |= p.WKy;
   i <<= 3;
   i |= p.BKx;
   i <<= 3;
   i |= p.BKy;
   i <<= 3;
   i |= p.WRx;
   i <<= 3;
   i |= p.WRy;
   return i;
}

/* -------------------------------------------------------------------------------- */

void Pos2Cartesian(int i,Position *p)  {
   p->WRy = i & 7;
   i >>= 3;
   p->WRx = i & 7;
   i >>= 3;
   p->BKy = i & 7;
   i >>= 3;
   p->BKx = i & 7;
   i >>= 3;
   p->WKy = i & 7;
   i >>= 3;
   p->WKx = i & 7;
}

/* -------------------------------------------------------------------------------- */

void Print(Position p) {
    int x,y;
    for(y=7;y>=0;y--) {
        for(x=0;x<8;x++)  {
            if(p.WKx==x && p.WKy==y)
                printf("|K");
            else if (p.BKx==x && p.BKy==y)
                printf("|k");
            else if (p.WRx==x && p.WRy==y)
                printf("|R");
            else 
                printf("| ");
        }
        printf("| \n");
    }
    printf("\n");
}

/* -------------------------------------------------------------------------------- */

int ManhattanDistance(int x1,int y1,int x2,int y2) {
    return AbsDiff(x1,x2)+AbsDiff(y1,y2);
}

/* -------------------------------------------------------------------------------- */

int MinusPA (int x, int y) {
    if (x<=y) 
        return 0;
    else 
        return x-y;
}

/* -------------------------------------------------------------------------------- */

int AbsDiff(int x, int y) {
    return MinusPA(x,y)+MinusPA(y,x);
}

/* -------------------------------------------------------------------------------- */

int Distance (int x1,int y1,int x2,int y2) {
    if (MinusPA(AbsDiff(y1,y2),AbsDiff(x1,x2))==0)
        return AbsDiff(x1,x2);
    else 
        return AbsDiff(y1,y2);
}

/* -------------------------------------------------------------------------------- */

Position Symmetric(int i, Position p) {
    Position pp=p;
    switch(i) {
      case 0: break;
      case 1: pp.WKx=7-p.WKx;pp.WRx=7-p.WRx;pp.BKx=7-p.BKx;break;
      case 2: pp.WKy=7-p.WKy;pp.WRy=7-p.WRy;pp.BKy=7-p.BKy;break;
      case 3: pp.WKx=7-p.WKx;pp.WRx=7-p.WRx;pp.BKx=7-p.BKx;
              pp.WKy=7-p.WKy;pp.WRy=7-p.WRy;pp.BKy=7-p.BKy;break;
      case 4: pp.WKx=p.WKy;pp.WRx=p.WRy;pp.BKx=p.BKy;
              pp.WKy=p.WKx;pp.WRy=p.WRx;pp.BKy=p.BKx;      break;
      case 5: pp.WKx=7-p.WKy;pp.WRx=7-p.WRy;pp.BKx=7-p.BKy;
              pp.WKy=p.WKx;pp.WRy=p.WRx;pp.BKy=p.BKx;      break;
      case 6: pp.WKx=p.WKy;pp.WRx=p.WRy;pp.BKx=p.BKy;
              pp.WKy=7-p.WKx;pp.WRy=7-p.WRx;pp.BKy=7-p.BKx;break;
      case 7: pp.WKx=7-p.WKy;pp.WRx=7-p.WRy;pp.BKx=7-p.BKy;
              pp.WKy=7-p.WKx;pp.WRy=7-p.WRx;pp.BKy=7-p.BKx;break;
      default: break;
    }
    return pp;
}


/* ******************************************************************** */
/* Chess rules for legal positions                                      */      
/* ******************************************************************** */

bool Dim(Position p) {
    return (p.WKx>=0 && p.WKx<=7 && p.WKy>=0 && p.WKy<=7 && 
            p.BKx>=0 && p.BKx<=7 && p.BKy>=0 && p.BKy<=7 &&
            p.WRx>=0 && p.WRx<=7 && p.WRy>=0 && p.WRy<=7);
}

/* -------------------------------------------------------------------------------- */

bool NotOnSameSquare (Position p) {
    // Rook and Black king may be on the same square if the black kind captured the rook
    return (p.WKx!=p.WRx || p.WKy!=p.WRy);
}

/* -------------------------------------------------------------------------------- */

bool NotKingNextKing (Position p) {
    return (p.WKx>p.BKx+1 || p.BKx>p.WKx+1 || p.WKy>p.BKy+1 || p.BKy>p.WKy+1);
}

/* -------------------------------------------------------------------------------- */

bool BlackKingAttacked (Position p) {
    if (RookCaptured(p)) return false;
    return (p.WRx==p.BKx && (p.WKx!=p.WRx || p.WKx==p.WRx && (p.WKy<=p.BKy && p.WKy<=p.WRy || p.BKy<=p.WKy && p.WRy<=p.WKy)) ||
            p.WRy==p.BKy && (p.WKy!=p.WRy || p.WKy==p.WRy && (p.WKx<=p.BKx && p.WKx<=p.WRx || p.BKx<=p.WKx && p.WRx<=p.WKx)));
}

/* -------------------------------------------------------------------------------- */

bool RookCaptured(Position p) {
    return (p.WRx==p.BKx && p.WRy==p.BKy);
}

/* -------------------------------------------------------------------------------- */

bool LegalPosWhiteToMove(Position p)  {
    return Dim(p) && NotKingNextKing(p) &&  NotOnSameSquare(p) && !BlackKingAttacked(p);
}

/* -------------------------------------------------------------------------------- */

bool LegalPosBlackToMove(Position p)  {
    return Dim(p) && NotKingNextKing(p) &&  NotOnSameSquare(p);
}

/* ******************************************************************** */
/* Chess rules for legal moves                                          */      
/* ******************************************************************** */

bool WhiteKingMoves(Position p1,Position p2) {
    return (p1.WKx!=p2.WKx || p1.WKy!=p2.WKy);
}

/* -------------------------------------------------------------------------------- */

bool WhiteRookMoves(Position p1,Position p2) {
    return (p1.WRx!=p2.WRx || p1.WRy!=p2.WRy);
}

/* -------------------------------------------------------------------------------- */

bool LegalWhiteMove(Position p1, Position p2)  {
    if (p1.BKx!=p2.BKx || p1.BKy!=p2.BKy) return false;
    
    if (!RookCaptured(p1) && p1.WKx==p2.WKx && p1.WKy==p2.WKy) { // Rook moves
        if (p1.WRx==p2.WRx && p1.WRy==p2.WRy) return false;     
        if (p1.WRx!=p2.WRx && p1.WRy!=p2.WRy) return false;
        if (p1.WRx>p2.WRx) {
            if (p1.WKy==p1.WRy && p1.WRx>p1.WKx && p1.WKx>p2.WRx) return false;
            if (p1.BKy==p1.WRy && p1.WRx>p1.BKx && p1.BKx>p2.WRx) return false; 
        }
        if (p1.WRx<p2.WRx) {
            if (p1.WKy==p1.WRy && p1.WRx<p1.WKx && p1.WKx<p2.WRx) return false; 
            if (p1.BKy==p1.WRy && p1.WRx<p1.BKx && p1.BKx<p2.WRx) return false;
        }
        if (p1.WRy>p2.WRy) {
            if (p1.WKx==p1.WRx && p1.WRy>p1.WKy && p1.WKy>p2.WRy) return false; 
            if (p1.BKx==p1.WRx && p1.WRy>p1.BKy && p1.BKy>p2.WRy) return false; 
        }
        if (p1.WRy<p2.WRy) {
            if (p1.WKx==p1.WRx && p1.WRy<p1.WKy && p1.WKy<p2.WRy) return false; 
            if (p1.BKx==p1.WRx && p1.WRy<p1.BKy && p1.BKy<p2.WRy) return false; 
        }
    }
    else { // King moves
        if (p1.WKx==p2.WKx && p1.WKy==p2.WKy) return false;     
        if (p1.WRx!=p2.WRx || p1.WRy!=p2.WRy) return false;
        if (p1.WKx>p2.WKx+1 || p2.WKx>p1.WKx+1 ||
            p1.WKy>p2.WKy+1 || p2.WKy>p1.WKy+1) return false;
    }
    return LegalPosBlackToMove(p2);
}

/* -------------------------------------------------------------------------------- */

bool LegalBlackMove(Position p1, Position p2)  {
    if (p1.WKx!=p2.WKx || p1.WKy!=p2.WKy) 
        return false;
    if (!RookCaptured(p2) && (p1.WRx!=p2.WRx || p1.WRy!=p2.WRy)) 
        return false;
    if (RookCaptured(p2) && (p2.WRx!=p2.BKx || p2.WRy!=p2.BKy)) 
        return false;
    if (p1.BKx==p2.BKx && p1.BKy==p2.BKy) 
        return false;       
    if (p1.BKx>p2.BKx+1 || p2.BKx>p1.BKx+1 || p1.BKy>p2.BKy+1 || p2.BKy>p1.BKy+1)   
        return false;
    return LegalPosWhiteToMove(p2);
}

/* -------------------------------------------------------------------------------- */

bool NextLegalWhiteMove(unsigned i,Position p1,Position *p2)  {
    *p2 = p1;
    switch(i) {
        case 0: p2->WKx = p2->WKx+1; break;
        case 1: p2->WKx = p2->WKx+1; p2->WKy = p2->WKy+1; break;
        case 2:                      p2->WKy = p2->WKy+1; break;
        case 3: p2->WKx = p2->WKx-1; p2->WKy = p2->WKy+1; break;
        case 4: p2->WKx = p2->WKx-1; break;
        case 5: p2->WKx = p2->WKx-1; p2->WKy = p2->WKy-1; break;
        case 6:                      p2->WKy = p2->WKy-1; break;
        case 7: p2->WKx = p2->WKx+1; p2->WKy = p2->WKy-1; break;

        case 8:
        case 9:
        case 10:
        case 11:
        case 12:
        case 13:
        case 14:
        case 15: if (!RookCaptured(p1)) p2->WRx = i-8; break;
        
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 21:
        case 22:
        case 23: if (!RookCaptured(p1)) p2->WRy = i-16; break;
        
        default: return false;
    }
    return LegalWhiteMove(p1, *p2);
}

/* -------------------------------------------------------------------------------- */

bool NextLegalBlackMove(unsigned i,Position p1,Position *p2)  {
    *p2 = p1;
    switch(i) {
        case 0: p2->BKx = p2->BKx+1; break;
        case 1: p2->BKx = p2->BKx+1; p2->BKy = p2->BKy+1; break;
        case 2:                      p2->BKy = p2->BKy+1; break;
        case 3: p2->BKx = p2->BKx-1; p2->BKy = p2->BKy+1; break;
        case 4: p2->BKx = p2->BKx-1; break;
        case 5: p2->BKx = p2->BKx-1; p2->BKy = p2->BKy-1; break;
        case 6:                      p2->BKy = p2->BKy-1; break;
        case 7: p2->BKx = p2->BKx+1; p2->BKy = p2->BKy-1; break;
        default: return false;
    }
    if (p1.BKx==p1.WRx && p1.BKy==p1.WRy) { 
        p2->WRx=p2->BKx; 
        p2->WRy=p2->BKy; 
    }
    return LegalBlackMove(p1, *p2);
}

/* ******************************************************************** */ 
/* Mate, Stalemate, Draw                                                */
/* ******************************************************************** */

bool StalemateOneCase(Position p) {
    return (p.BKx==0 && p.BKy==0 && p.WRx==1 && p.WRy==1 && p.WKx==2 && p.WKy<3 ||
            p.BKx==0 && p.BKy==0 && p.WRx>0 && p.WRy==1 && p.WKx==2 && p.WKy==0);
}

/* -------------------------------------------------------------------------------- */

bool Stalemate(Position p) {
    int i;
    for(i=0;i<8;i++)
        if(StalemateOneCase(Symmetric(i,p)))
            return true;
    return false;
}

/* ******************************************************************** */
/* Definitions for Strategy                                             */
/* ******************************************************************** */

int Room (Position p) {
    if (AbsDiff(p.WRx,p.BKx)==0)
        return 15;
    else {
        if (AbsDiff(p.WRy,p.BKy)==0)
            return 15;
        else {
            if (MinusPA(p.BKx,p.WRx)==0) {
                if (MinusPA(p.BKy,p.WRy)==0)
                    return p.WRx+p.WRy;
                else 
                    return p.WRx+(7-p.WRy);
            }
            else {
                if (MinusPA(p.BKy,p.WRy)==0)
                    return (7-p.WRx)+p.WRy;
                else 
                    return (7-p.WRx)+(7-p.WRy);
            }
        }
    }
}

/* -------------------------------------------------------------------------------- */

int RoomM (Position p) {
    if (p.WRx==p.BKx)
        return 64;
    if (p.WRy==p.BKy)
        return 64;
    if (p.BKx<=p.WRx) {
        if (p.BKy<=p.WRy)
            return p.WRx*p.WRy;
        else 
            return p.WRx*(7-p.WRy);
    }
    else {
        if (p.BKy<=p.WRy)
            return (7-p.WRx)*p.WRy;
        else 
            return (7-p.WRx)*(7-p.WRy);
    }
}

/* -------------------------------------------------------------------------------- */

bool NewRoomSmaller (Position p1,Position p2) {
    return (Room(p1)>Room(p2));
}

/* -------------------------------------------------------------------------------- */

bool NotBlackKingNextRook (Position p)  {
    return (p.WRx>p.BKx+ 1 || p.BKx>p.WRx+1 || p.WRy>p.BKy+1 || p.BKy>p.WRy+1);
}

/* -------------------------------------------------------------------------------- */

bool WhiteKingNextRook (Position p)  {
    return (p.WRx<=p.WKx+1 && p.WKx<=p.WRx+1 && p.WRy<=p.WKy+1 && p.WKy<=p.WRy+1);
}

/* -------------------------------------------------------------------------------- */

int DistanceWhiteKingToCS (Position p) {
    return Distance(p.WKx,p.WKy,CSx(p),CSy(p));
}

/* -------------------------------------------------------------------------------- */

int DistanceWhiteKingToRook (Position p) {
    return Distance(p.WKx,p.WKy,p.WRx,p.WRy);
}

/* -------------------------------------------------------------------------------- */

int DistanceBlackKingToRook (Position p) {
    return Distance(p.BKx,p.BKy,p.WRx,p.WRy);
}

/* -------------------------------------------------------------------------------- */

int DistanceWhiteKingToBlackKing (Position p) {
    return Distance(p.WKx,p.WKy,p.BKx,p.BKy);
}

/* -------------------------------------------------------------------------------- */

bool NotWhiteRookExposed (Position p) {
    return (DistanceWhiteKingToRook(p)<=DistanceBlackKingToRook(p));
}

/* -------------------------------------------------------------------------------- */

bool WhiteRookDivides (Position p) {
    return (p.WKx>p.WRx && p.WRx>p.BKx || p.WKx<p.WRx && p.WRx<p.BKx || 
            p.WKy>p.WRy && p.WRy>p.BKy || p.WKy<p.WRy && p.WRy<p.BKy);
}

/* -------------------------------------------------------------------------------- */

int CSx(Position p) {
    if (AbsDiff(p.BKx,p.WRx)== 0) 
        return p.WRx;
    else {
        if (MinusPA(p.BKx,p.WRx) == 0) 
            return p.WRx-1;
        else 
            return p.WRx+1;
    }
}

/* -------------------------------------------------------------------------------- */

int CSy(Position p) {
    if (AbsDiff(p.BKy,p.WRy) == 0) 
        return p.WRy;
    else {
        if (MinusPA(p.BKy,p.WRy) == 0) 
            return p.WRy-1;
        else 
            return p.WRy+1;
    }
}

/* -------------------------------------------------------------------------------- */

bool ApproachCriticalSquare (Position p1,Position p2) {
    int d1=ManhattanDistance(p1.WKx,p1.WKy,CSx(p1),CSy(p1));
    int d2=ManhattanDistance(p2.WKx,p2.WKy,CSx(p2),CSy(p2));
    return (d1>d2);
}

/* -------------------------------------------------------------------------------- */

bool WhiteKingEdge(Position p) {
    return (p.WKx==0 || p.WKx==7 || p.WKy==0 || p.WKy==7);
}

/* -------------------------------------------------------------------------------- */

bool LPattern(Position p) {
    return (p.WKy==p.BKy && (p.WKx==p.BKx+2 || p.BKx==p.WKx+2) && 
            p.WRx==p.WKx && (p.WRy==p.WKy+1 || p.WKy==p.WRy+1)||
            p.WKx==p.BKx && (p.WKy==p.BKy+2 || p.BKy==p.WKy+2) && 
            p.WRy==p.WKy && (p.WRx==p.WKx+1 ||  p.WKx==p.WRx+1));
}

/* -------------------------------------------------------------------------------- */

bool WhiteKingAndRookNotDiverging (Position p1,Position p2) {
    return (DistanceWhiteKingToRook(p1) >= DistanceWhiteKingToRook(p2));
}

/* -------------------------------------------------------------------------------- */

/* ******************************************************************** */
/* Strategy (for White)                                                 */
/* ******************************************************************** */


bool ImmediateMateCond(Position p1,Position p2) {
    int kk;
    Position p3;
    if(Stalemate(p2))
        return false;

    if(!BlackKingAttacked(p2)) 
        return false;
    
    bool bEscape=false;
    for(kk=0;kk<8;kk++)
       if (NextLegalBlackMove(kk,p2,&p3))    
          return false;

    return true;
}

/* -------------------------------------------------------------------------------- */


bool ReadyToMateCond(Position p1,Position p2) {
    int i;
    Position pp2;
    if(Stalemate(p2))
        return false;

    for(i=0;i<8;i++) {
        pp2=Symmetric(i,p2);
        if(pp2.WKx==2 && pp2.BKx==0 && pp2.BKy==1 && pp2.WRx>1 && pp2.WRy==2 && (pp2.WKy==1 || pp2.WKy==0))
            return true;

        /* Alternative:
        if(pp2.BKy==0 && pp2.WKy==2 && pp2.WKx==pp2.BKx-1 && pp2.WRy>1 && pp2.WRx==pp2.BKx+1)
            return true;
        if(pp2.BKy==0 && pp2.WKy==2 && pp2.BKx==1 && pp2.WKx==1 && pp2.WRy>0 && pp2.WRx==2)
            return true;
            */
    }
    return false;
}

/* -------------------------------------------------------------------------------- */


bool SqueezeCond(Position p1,Position p2) {
    int j;
    if(Stalemate(p2)) 
        return false;
    if(!WhiteRookMoves(p1,p2))
        return false;

    if (!(NewRoomSmaller(p1,p2) && NotWhiteRookExposed(p2) && WhiteRookDivides(p2)))
        return false;

    Position p3;
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p3) && !Stalemate(p3) && WhiteRookMoves(p1,p3))  
            if (NewRoomSmaller(p2,p3) && NotWhiteRookExposed(p3) && WhiteRookDivides(p3))
                return false;

    return true;
}

/* -------------------------------------------------------------------------------- */

bool ApproachCondDiag(Position p1,Position p2) {
    if(Stalemate(p2)) 
        return false;
    if(!WhiteKingMoves(p1,p2))
        return false;
    if(p2.WKx==p1.WKx || p2.WKy==p1.WKy)
        return false;
    return (ApproachCriticalSquare(p1,p2) && 
            NotWhiteRookExposed(p2) &&
           (WhiteRookDivides(p2) || LPattern(p2)) && 
           (Room(p2) > 3 || !WhiteKingEdge(p2)));
}

/* -------------------------------------------------------------------------------- */

bool ApproachCondNonDiag(Position p1,Position p2) {
    if(Stalemate(p2)) 
        return false;
    if(!WhiteKingMoves(p1,p2))
        return false;
    if(p2.WKx!=p1.WKx && p2.WKy!=p1.WKy)
        return false;
    return (ApproachCriticalSquare(p1,p2) &&    
            NotWhiteRookExposed(p2) &&
            (WhiteRookDivides(p2) || LPattern(p2)) && 
            (Room(p2) > 3 || ! WhiteKingEdge(p2)));
}

/* -------------------------------------------------------------------------------- */

bool KeepRoomCondDiag(Position p1,Position p2) {
    if(Stalemate(p2)) 
        return false;
    if(!WhiteKingMoves(p1,p2))
        return false;
    if(p2.WKx==p1.WKx || p2.WKy==p1.WKy)
        return false;
    return (NotWhiteRookExposed(p2) &&
            WhiteRookDivides(p2) &&
            WhiteKingAndRookNotDiverging(p1,p2) &&
            (Room(p2) > 3 || ! WhiteKingEdge(p2)));
}

/* -------------------------------------------------------------------------------- */

bool KeepRoomCondNonDiag(Position p1,Position p2) {
    if(Stalemate(p2)) 
        return false;
    if(!WhiteKingMoves(p1,p2))
        return false;
    if(p2.WKx!=p1.WKx && p2.WKy!=p1.WKy)
        return false;
    return (NotWhiteRookExposed(p2) &&
            WhiteRookDivides(p2) &&
            WhiteKingAndRookNotDiverging(p1,p2) &&
            (Room(p2) > 3 || ! WhiteKingEdge(p2)));
}

/* -------------------------------------------------------------------------------- */


bool RookHomeCond(Position p1,Position p2) {
    if (Stalemate(p2))
        return false;
    if(!WhiteRookMoves(p1,p2))
        return false;
    if (!NotBlackKingNextRook(p2) && !WhiteKingNextRook(p2))
        return false;
        return
        (p1.BKx<p1.WKx && p1.WRx!=p1.WKx-1 && p2.WRx==p1.WKx-1 ||
         p1.BKx>p1.WKx && p1.WRx!=p1.WKx+1 && p2.WRx==p1.WKx+1 ||
         p1.BKy<p1.WKy && p1.WRy!=p1.WKy-1 && p2.WRy==p1.WKy-1 ||
         p1.BKy>p1.WKy && p1.WRy!=p1.WKy+1 && p2.WRy==p1.WKy+1 ||
         p1.WRx==p1.WKx && p1.WKx==p1.BKx && (p2.WRx==p1.WKx+1 || p2.WRx==p1.WKx-1) ||
         p1.WRy==p1.WKy && p1.WKy==p1.BKy && (p2.WRy==p1.WKy+1 || p2.WRy==p1.WKy-1));
}

/* -------------------------------------------------------------------------------- */

bool RookSafeCond(Position p1,Position p2) {
    if (Stalemate(p2))
        return false;
    if(!WhiteRookMoves(p1,p2))
        return false;
	if ((DistanceBlackKingToRook(p2)<=2) && 
        (NotBlackKingNextRook(p2) || !WhiteKingNextRook(p2))) 
       return false;
    return (((p1.WRx!=0 && p2.WRx==0) ||
             (p1.WRx!=7 && p2.WRx==7) ||
             (p1.WRy!=0 && p2.WRy==0) ||
             (p1.WRy!=7 && p2.WRy==7)));
}


/* -------------------------------------------------------------------------------- */

Position Strategy(Position p1, eStratStep *s) {
    int j;
    Position p2; 
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(ImmediateMateCond(p1,p2)) {
                *s=eImmediateMate;    
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(ReadyToMateCond(p1,p2)) {
                *s=eReadyToMate;    
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(SqueezeCond(p1,p2)) {
                *s=eSqueeze;    
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(ApproachCondDiag(p1,p2)) {
                *s=eApproachDiag;   
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(ApproachCondNonDiag(p1,p2)) {
                *s=eApproachNonDiag;    
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(KeepRoomCondDiag(p1,p2)) {
                *s=eKeepRoomDiag;   
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(KeepRoomCondNonDiag(p1,p2)) {
                *s=eKeepRoomNonDiag;    
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(RookHomeCond(p1,p2)) {
                *s=eRookHome;   
                return p2;
            }
    for(j=0;j<24;j++)  
        if (NextLegalWhiteMove(j,p1,&p2))  
            if(RookSafeCond(p1,p2)) {
                *s=eRookSafe;
                return p2;
            }

    return p1;
}


/* -------------------------------------------------------------------------------- */

// ********************************************************************
// Search procedures -- checking correctness, finding optimal moves
// ********************************************************************


void Init() {
    Position p1;
    int i,PosNo=0;
    
    for(i=0;i<MAX_POS;i++) {
        Pos2Cartesian(i,&p1);
        if (!LegalPosWhiteToMove(p1))  {
            LookUpTable[i].OptimalDist=ILEGAL;
            LookUpTable[i].StrategDist=ILEGAL;
        }
        else  {
            if (RookCaptured(p1)) { 
                LookUpTable[i].OptimalDist=DRAW;
                LookUpTable[i].StrategDist=DRAW;
            }
            else {
                LookUpTable[i].OptimalDist=UNKNOWN;
                LookUpTable[i].StrategDist=UNKNOWN;
                PosNo++;
            }
        }
    }
    printf("Total number of legal positions: %i.\n",PosNo);

    SStep[eImmediateMate]=  "ImmediateMate  ";
    SStep[eReadyToMate]=    "ReadyToMate    ";
    SStep[eSqueeze]=        "Squeeze        ";
    SStep[eApproachDiag]=   "ApproachDiag   ";
    SStep[eApproachNonDiag]="ApproachNonDiag";
    SStep[eKeepRoomDiag]=   "KeepRoomDiag   ";
    SStep[eKeepRoomNonDiag]="KeepRoomNonDiag";
    SStep[eRookHome]=       "RookHome       ";
    SStep[eRookSafe]=       "RookSafe       ";

    _B_INITIALIZATION_DONE=true;
}

/* -------------------------------------------------------------------------------- */

void SetStrategyMoves() {
    int i;
    Position p;
    eStratStep s;
    
    if (!_B_INITIALIZATION_DONE)
        Init();

    for(i=0;i<MAX_POS;i++) 
        if (LookUpTable[i].StrategDist!=ILEGAL && LookUpTable[i].StrategDist!=DRAW)  {
            Pos2Cartesian(i,&p);
            LookUpTable[i].StrategPos = Cartesian2Pos(Strategy(p,&s));
            LookUpTable[i].step = s;
        }

    _B_SETTING_STEPS_DONE=true;
}


/* -------------------------------------------------------------------------------- */

void ComputeOptimalDistancesToWin() {
    int i,j,k;
    Position p1,p2,p3;

    if (!_B_INITIALIZATION_DONE)
        Init();

    int Iteration=0,Updated=0;
    do {
        Updated=0;
        Iteration++;
        printf("Iteration: %i.\n",Iteration);

        for(i=0;i<MAX_POS;i++) {

            if (LookUpTable[i].OptimalDist!=ILEGAL && LookUpTable[i].OptimalDist!=DRAW)  {
                Pos2Cartesian(i,&p1);

                for(j=0;j<24;j++) { // find best move for White 
                    if (NextLegalWhiteMove(j,p1,&p2)) { 

                        int d3_max=UNDEF;
                        bool bNoLegalMoves=true;
                        for(k=0;k<8;k++)  {  // find best move for Black 
                            if (NextLegalBlackMove(k,p2,&p3))  {  
                                bNoLegalMoves=false;
                                int d3_cur = LookUpTable[Cartesian2Pos(p3)].OptimalDist;
                                if (d3_cur==DRAW || d3_cur==UNKNOWN)
                                    d3_max=d3_cur;
                                else if (d3_max==UNDEF || d3_cur>d3_max) 
                                    d3_max=d3_cur;
                            }
                        }

                        if (bNoLegalMoves) {
                            if(BlackKingAttacked(p2) && LookUpTable[i].OptimalDist!=1) { // mate
                                LookUpTable[i].OptimalDist=1;
                                LookUpTable[i].OptimalPos=Cartesian2Pos(p2);;
                                Updated++;
                            }
                        }
                        else if(d3_max!=ILEGAL && d3_max!=UNKNOWN && d3_max!=DRAW && LookUpTable[i].OptimalDist>d3_max+2) {
                            LookUpTable[i].OptimalDist = d3_max+2;
                            LookUpTable[i].OptimalPos=Cartesian2Pos(p2);;
                            Updated++;
                        }
                    }
                }
            }
        }

        int Dist[1010];
        for(i=0;i<1000;i++) 
            Dist[i]=0;
        for(i=0;i<MAX_POS;i++) 
            Dist[LookUpTable[i].OptimalDist]++;
        for(i=0;i<50;i++) 
            if (i%2)
                printf("Guaranteed mate in: %i plies for: %i positions.\n",i,Dist[i]);

        int u=0;
        for(i=0;i<MAX_POS;i++) 
            if (LookUpTable[i].OptimalDist==UNKNOWN)
                u++;
        printf("Unknown: %i.\n", u);
        printf("Updated: %i.\n", Updated);
    }
    while(Updated);
    getc(stdin);
}

/* -------------------------------------------------------------------------------- */

void ComputeStrategyDistancesToWin() {
    Position p,np,p1,p2,p3;
    eStratStep s;
    int i,k,u,d3_max,Iteration=0,Updated=0;

    if (!_B_SETTING_STEPS_DONE)
        SetStrategyMoves();

    do {
        Updated=0;
        Iteration++;
        printf("Iteration: %i\n", Iteration);

        for(i=0;i<MAX_POS;i++) {
            if (LookUpTable[i].StrategDist!=ILEGAL && LookUpTable[i].StrategDist!=DRAW)  {
                Pos2Cartesian(i,&p1);
                Pos2Cartesian(LookUpTable[i].StrategPos,&p2);
                d3_max=UNDEF;
                bool bNoLegalMoves=true;
                for(k=0;k<8;k++)  {  // find best move for Black 
                    if (NextLegalBlackMove(k,p2,&p3))  {  
                        bNoLegalMoves=false;
                        int d3_cur = LookUpTable[Cartesian2Pos(p3)].StrategDist;
                        if (d3_cur==DRAW || d3_cur==UNKNOWN)
                            d3_max=d3_cur;
                        else if (d3_max==UNDEF || d3_cur>d3_max) 
                            d3_max=d3_cur;
                    }
                }

                if (bNoLegalMoves) {
                    if(BlackKingAttacked(p2) && LookUpTable[i].StrategDist!=1) { // mate
                        LookUpTable[i].StrategDist=1;
                        Updated++;
                    }
                }
                else if(d3_max!=ILEGAL && d3_max!=UNKNOWN && d3_max!=DRAW && LookUpTable[i].StrategDist>d3_max+2) {
                    LookUpTable[i].StrategDist = d3_max+2;
                    Updated++;
                }
            }
        }

        int Dist[1010];
        for(i=0;i<1000;i++) 
            Dist[i]=0;
        for(i=0;i<MAX_POS;i++) 
            Dist[LookUpTable[i].StrategDist]++;
        for(i=0;i<100;i++) 
            if (i%2)
                printf("Guaranteed mate in: %i plies for: %i positions.\n",i,Dist[i]);

        u=0;
        for(i=0;i<MAX_POS;i++) 
            if (LookUpTable[i].StrategDist==UNKNOWN)
                u++;
        printf("Unknown: %i.\n", u);
        printf("Updated: %i.\n", Updated);
    }
    while(Updated);

    if(u==0)
        printf("Strategy is correct!\n");
    else
        printf("Strategy is incorrect!\n");

    for(i=0;i<MAX_POS;i++) 
        if (LookUpTable[i].StrategDist==UNKNOWN)  {
            printf("Position UNKNOWN %i\n", i);;
            Pos2Cartesian(i,&p);
            Print(p);
            np=Strategy(p,&s);  
            printf("Strategy move (%i): \n",LookUpTable[i].step);
            Print(np);
            printf("Press Enter... \n");
            getc(stdin);
        }
}

/* -------------------------------------------------------------------------------- */

void MeasureDecreases() {
    Position p,pm,po; 
    eStratStep s;
    int i,m1,mo,mdec=0,opt=0,im;

    if (!_B_INITIALIZATION_DONE)
        Init();
    
    for(i=0;i<MAX_POS;i++) 
        if (LookUpTable[i].OptimalDist!=ILEGAL && LookUpTable[i].OptimalDist!=DRAW)  {
            Pos2Cartesian(i,&p);
            m1=RoomM(p);
            pm=Strategy(p,&s);
            im=Cartesian2Pos(pm);
            Pos2Cartesian(LookUpTable[i].OptimalPos,&po);
            mo=RoomM(po);
            if (m1>mo) mdec++;
            if(im==LookUpTable[i].OptimalPos) opt++;
            else {
                printf("Starting position : %i",i);
                Print(p);
                printf("Strategy\n");
                Print(pm);
                printf("Optimal\n");
                Print(po);
            }
        }
        printf("The measure decreases in %i cases\n",mdec);
        printf("Optimal move chosen in %i cases\n",opt);
}

/* -------------------------------------------------------------------------------- */

void CountStrategySteps() {
    int i,Steps[eStepMax],Total=0;

    if (!_B_SETTING_STEPS_DONE)
        SetStrategyMoves();
    
    for(i=0;i<eStepMax;i++) 
        Steps[i]=0;
    for(i=0;i<MAX_POS;i++) 
        if (LookUpTable[i].OptimalDist!=ILEGAL && LookUpTable[i].OptimalDist!=DRAW) { 
            Steps[LookUpTable[i].step]++;
            Total++;
        }
    for(i=0;i<eStepMax;i++)  
        printf("Step : %s : %i\n", SStep[i], Steps[i]);
    printf("----------------------------------------------\n");
    printf("Total:                     %i\n",Total);
}

/* -------------------------------------------------------------------------------- */

int Measure(Position p) {
    return 6*Room(p)+ManhattanDistance(p.WKx,p.WKy,CSx(p),CSy(p));
}

/* -------------------------------------------------------------------------------- */

// ********************************************************************
// Main
// ********************************************************************

int main() {
    time_t seconds_start,seconds_curr;
    seconds_start = time (NULL);

    _B_INITIALIZATION_DONE=false;
    _B_SETTING_STEPS_DONE=false;

    // ComputeOptimalDistancesToWin();
    // MeasureDecreases();
      SetStrategyMoves();
    CountStrategySteps();
    // CountMeasureDecreasing();
    ComputeStrategyDistancesToWin();
    CountStrategySteps();

    seconds_curr = time (NULL);
    printf("Time spent: %u seconds\n", (unsigned int)(seconds_curr-seconds_start));

    printf("Press Enter... \n");
    getc(stdin);
    return 0;
}
