/* ******************************* URSA-krk ************************************ */

/* ***************************************************************************** */
/* ***************************************************************************** */
/* *************** Specification of Chess Rules for KRK ending ***************** */
/* ***************************************************************************** */
/* ***************************************************************************** */

/* ************* Written by Marko Malikovic and Predrag Janicic (2013)********** */

/* ***************************************************************************** */
/* For convenience, a KRK position is represented as a sequence of 19 bits:
   ----------------------------------------------------------
   |18|17|16|15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0|
   ----------------------------------------------------------
   |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
   ----------------------------------------------------------
   |bW|  nWRy  |  nWRx  |  nBKy  |  nBKx  |  nWKy  |  nWKx  |    
   ----------------------------------------------------------  */
/* ***************************************************************************** */


/* ******************************************************************** */
/* Miscellanous                                                         */ 
/* ******************************************************************** */ 

/* -------------------------------------------------------------------------------- */

/* Compressing and uncompressing procedures */

procedure Cartesian2Pos(nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn, nPos) {
  nPos = ite(bWhiteOnTurn,1,0);
  nPos = (nPos << 3) | (nWRy & 7);
  nPos = (nPos << 3) | (nWRx & 7);
  nPos = (nPos << 3) | (nBKy & 7);
  nPos = (nPos << 3) | (nBKx & 7);
  nPos = (nPos << 3) | (nWKy & 7);
  nPos = (nPos << 3) | (nWKx & 7);
}

procedure Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn) {
  nWKx = nPos & 7;
  nWKy = nPos >>  3 & 7;
  nBKx = nPos >>  6 & 7;
  nBKy = nPos >>  9 & 7;
  nWRx = nPos >> 12 & 7;
  nWRy = nPos >> 15 & 7;
  bWhiteOnTurn = num2bool(nPos >> 18);
}

procedure IsWhiteOnTurn(nPos, bWhiteOnTurn) {
  bWhiteOnTurn = num2bool(nPos >> 18);
}

/* -------------------------------------------------------------------------------- */

procedure ManhattanDistance(nx1, ny1, nx2, ny2, nMD) {
  nMD = ite(nx2>=nx1, 
            ite(ny2>=ny1, (nx2-nx1)+(ny2-ny1), (nx2-nx1)+(ny1-ny2)),
            ite(ny2>=ny1, (nx1-nx2)+(ny2-ny1), (nx1-nx2)+(ny1-ny2)));
}

procedure Max(nx, ny, nMax) {
  nMax = ite(nx>=ny, nx, ny);
}

procedure ChebyshevDistance(nx1, ny1, nx2, ny2, nCD) {
call Max(nx2-nx1, ny2-ny1, nMax1);
call Max(nx2-nx1, ny1-ny2, nMax2);
call Max(nx1-nx2, ny2-ny1, nMax3);
call Max(nx1-nx2, ny1-ny2, nMax4);
nCD = ite(nx2>=nx1,
          ite(ny2>=ny1, nMax1, nMax2),
          ite(ny2>=ny1, nMax3, nMax4));
}

/* Square (x2,y2) is between squares (x1,y1) and (x3,y3): */

procedure Between(nx1, ny1, nx2, ny2, nx3, ny3, bBetween) {
  bBetween = (nx1==nx2 && nx2==nx3 && ((ny1<ny2 && ny2<ny3) || (ny1>ny2 && ny2>ny3)))
             ||
             (ny1==ny2 && ny2==ny3 && ((nx1<nx2 && nx2<nx3) || (nx1>nx2 && nx2>nx3)));
}

/* ******************************************************************** */
/* Chess rules for legal positions                                      */      
/* ******************************************************************** */

procedure NotOnSameSquare(nPos, bNotOnSameSquare) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bNotOnSameSquare = (nWKx != nWRx) || (nWKy != nWRy);
}

procedure NotKingNextKing(nPos, bNotKingNextKing) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bNotKingNextKing = nWKx>nBKx+1 || nBKx>nWKx+1 || nWKy>nBKy+1 || nBKy>nWKy+1;
}

procedure BlackKingAttacked(nPos, bBlackKingAttacked) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call Between(nWRx, nWRy, nWKx, nWKy, nBKx, nBKy, bBetween);
  bBlackKingAttacked = (nWRx==nBKx ^^ nWRy==nBKy) && !bBetween;
}

procedure RookCaptured(nPos, bRookCaptured) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bRookCaptured = nWRx==nBKx && nWRy==nBKy;
}

procedure LegalKRKPosition(nPos, bLegalKRKPosition) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call NotOnSameSquare(nPos, bNotOnSameSquare);
  call NotKingNextKing(nPos, bNotKingNextKing);
  call BlackKingAttacked(nPos, bBlackKingAttacked);
  call RookCaptured(nPos, bRookCaptured);
  bLegalKRKPosition = bNotOnSameSquare && bNotKingNextKing && !bRookCaptured && !(bBlackKingAttacked && bWhiteOnTurn);
}

procedure LegalKRK_KKPosition(nPos, bLegalKRK_KKPosition) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call NotOnSameSquare(nPos, bNotOnSameSquare);
  call NotKingNextKing(nPos, bNotKingNextKing);
  call BlackKingAttacked(nPos, bBlackKingAttacked);
  bLegalKRK_KKPosition = bNotOnSameSquare && bNotKingNextKing && !(bBlackKingAttacked && bWhiteOnTurn);
}



/* ******************************************************************** */
/* Chess rules for legal moves                                          */      
/* ******************************************************************** */

procedure MoveWhiteKing(nPos1, nPos2, bMoveWhiteKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call ChebyshevDistance(nWKx1, nWKy1, nWKx2, nWKy2, nCD);
  bMoveWhiteKing = nCD==1;
}

procedure MoveBlackKing(nPos1, nPos2, bMoveBlackKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call ChebyshevDistance(nBKx1, nBKy1, nBKx2, nBKy2, nCD);
  bMoveBlackKing = (nCD==1);
}

procedure MoveWhiteRook(nPos1, nPos2, bMoveWhiteRook) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call Between(nWRx1, nWRy1, nWKx1, nWKy1, nWRx2, nWRy2, bBetween);
  bMoveWhiteRook = ((nWRx2==nWRx1 && nWRy2!=nWRy1) || (nWRx2!=nWRx1 && nWRy2==nWRy1)) && !bBetween;
}

/* Only one piece can move: */

procedure OtherAfterMoveWhiteKing(nPos1, nPos2, bOtherAfterMoveWhiteKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bOtherAfterMoveWhiteKing = (nBKx2==nBKx1 && nBKy2==nBKy1 && nWRx2==nWRx1 && nWRy2==nWRy1);
}

procedure OtherAfterMoveBlackKing(nPos1, nPos2, bOtherAfterMoveBlackKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bOtherAfterMoveBlackKing = (nWKx2==nWKx1 && nWKy2==nWKy1 && nWRx2==nWRx1 && nWRy2==nWRy1);
}

procedure OtherAfterMoveWhiteRook(nPos1, nPos2, bOtherAfterMoveWhiteRook) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bOtherAfterMoveWhiteRook = (nBKx2==nBKx1 && nBKy2==nBKy1 && nWKx2==nWKx1 && nWKy2==nWKy1);
}

/* -------------------------------------------------------------------------------- */

/* Rules for legal moves */

procedure LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call MoveWhiteKing(nPos1, nPos2, bMoveWhiteKing);
  call OtherAfterMoveWhiteKing(nPos1, nPos2, bOtherAfterMoveWhiteKing);
  call LegalKRKPosition(nPos2, bLegalKRKPosition2);
  bLegalMoveWhiteKing = bMoveWhiteKing && bOtherAfterMoveWhiteKing && bWhiteOnTurn1 && !bWhiteOnTurn2 && bLegalKRKPosition2;
}

procedure LegalMoveBlackKing(nPos1, nPos2, bLegalMoveBlackKing) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call MoveBlackKing(nPos1, nPos2, bMoveBlackKing);
  call OtherAfterMoveBlackKing(nPos1, nPos2, bOtherAfterMoveBlackKing);
  call LegalKRK_KKPosition(nPos2, bLegalKRK_KKPosition2);
  bLegalMoveBlackKing = bMoveBlackKing && bOtherAfterMoveBlackKing && !bWhiteOnTurn1 && bWhiteOnTurn2 && bLegalKRK_KKPosition2;
}

procedure LegalMoveWhiteRook(nPos1, nPos2, bLegalMoveWhiteRook) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call MoveWhiteRook(nPos1, nPos2, bMoveWhiteRook); 
  call OtherAfterMoveWhiteRook(nPos1, nPos2, bOtherAfterMoveWhiteRook);
  call LegalKRKPosition(nPos2, bLegalKRKPosition2);
  bLegalMoveWhiteRook = bMoveWhiteRook && bOtherAfterMoveWhiteRook && bWhiteOnTurn1 && !bWhiteOnTurn2 && bLegalKRKPosition2;
}

/* All possible legal moves of players: */

procedure LegalMoveWhite(nPos1, nPos2, bLegalMoveWhite) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing);
  call LegalMoveWhiteRook(nPos1, nPos2, bLegalMoveWhiteRook);
  bLegalMoveWhite = bLegalMoveWhiteKing || bLegalMoveWhiteRook;
}

procedure LegalMoveBlack(nPos1, nPos2, bLegalMoveBlack) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call LegalMoveBlackKing(nPos1, nPos2, bLegalMoveBlackKing);
  bLegalMoveBlack = bLegalMoveBlackKing;
}

/* ******************************************************************** */ 
/* Mate, Stalemate, Draw                                                */
/* ******************************************************************** */

procedure SquareAttackedByWhite(nPos, nX, nY, bSquareAttacked) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call Between(nWRx, nWRy, nWKx, nWKy, nX, nY, bBetween);
  call ChebyshevDistance(nWKx, nWKy, nX, nY, nCD);
  bSquareAttacked = (((nWRx==nX ^^ nWRy==nY) && !bBetween) || nCD==1);
}

procedure BlackKingCannotMove(nPos, bBlackKingCannotMove) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  nBKxInc =   nBKx+1;
  nBKyInc =   nBKy+1;
  nBKxDec =   nBKx-1;
  nBKyDec =   nBKy-1;
  bBlackKingCannotMove = !bWhiteOnTurn;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKy, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKyInc, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==7 || nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKyDec, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==7 || nBKy==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKy, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKyInc, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==0 || nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKyDec, bSquareAttacked);
  bBlackKingCannotMove &&= nBKx==0 || nBKy==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKx, nBKyInc, bSquareAttacked);
  bBlackKingCannotMove &&= nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKx, nBKyDec, bSquareAttacked);
  bBlackKingCannotMove &&= nBKy==0 || bSquareAttacked;
}

procedure ThreatenedByStalemate(nPos, bThreatenedByStalemate) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  nBKxInc =   nBKx+1;
  nBKyInc =   nBKy+1;
  nBKxDec =   nBKx-1;
  nBKyDec =   nBKy-1;
  bThreatenedByStalemate = bWhiteOnTurn;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKy, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKyInc, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==7 || nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxInc, nBKyDec, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==7 || nBKy==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKy, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKyInc, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==0 || nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKxDec, nBKyDec, bSquareAttacked);
  bThreatenedByStalemate &&= nBKx==0 || nBKy==0 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKx, nBKyInc, bSquareAttacked);
  bThreatenedByStalemate &&= nBKy==7 || bSquareAttacked;

  call SquareAttackedByWhite(nPos, nBKx, nBKyDec, bSquareAttacked);
  bThreatenedByStalemate &&= nBKy==0 || bSquareAttacked;
}

procedure Mate(nPos, bMate) {
  call BlackKingCannotMove(nPos, bBlackKingCannotMove);
  call BlackKingAttacked(nPos, bBlackKingAttacked);
  bMate = bBlackKingCannotMove && bBlackKingAttacked;
}

procedure Stalemate(nPos, bStalemate) {
  call BlackKingCannotMove(nPos, bBlackKingCannotMove);
  call BlackKingAttacked(nPos, bBlackKingAttacked);
  bStalemate = bBlackKingCannotMove && !bBlackKingAttacked;
}

procedure Draw(nPos, bDraw) {
  call IsWhiteOnTurn(nPos, bWhiteOnTurn);
  call RookCaptured(nPos,bRookCaptured);
  bDraw = bWhiteOnTurn && bRookCaptured;
}

/* ******************************************************************** */
/* Definitions for Strategy                                             */
/* ******************************************************************** */

procedure Room(nPos, nRoom) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  nRoom = ite (nWRx==nBKx, 15,
            ite (nWRy==nBKy, 15,
              ite (nWRx>nBKx,
                ite (nWRy>nBKy, nWRx+nWRy, nWRx+(7-nWRy)),
                  ite (nWRy>nBKy, (7-nWRx)+nWRy, (7-nWRx)+(7-nWRy)))));
}

/* The condition for the contraction of space of the black king with white rook: */

procedure NewRoomSmaller(nPos1, nPos2, bNewRoomSmaller) {
  call Room(nPos1, nRoom1);
  call Room(nPos2, nRoom2);
  bNewRoomSmaller = nRoom1>nRoom2;
}

/* White rook is not exposed to attack of the black king: */

procedure NotWhiteRookExposed(nPos, bNotWhiteRookExposed) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call ChebyshevDistance(nWKx, nWKy, nWRx, nWRy, nCDWKWR);
  call ChebyshevDistance(nBKx, nBKy, nWRx, nWRy, nCDBKWR);
  bNotWhiteRookExposed = (bWhiteOnTurn && nCDWKWR<=nCDBKWR+1) || (!bWhiteOnTurn && nCDWKWR<=nCDBKWR);
}

/* The kings are separated by a rook (either horizontally or vertically): */

procedure WhiteRookDivides(nPos, bWhiteRookDivides) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bWhiteRookDivides = (nWKx>nWRx && nWRx>nBKx) || (nWKx<nWRx && nWRx<nBKx) || (nWKy>nWRy && nWRy>nBKy) || (nWKy<nWRy && nWRy<nBKy);
}

/* Critical square: */

procedure CriticalSquare(nPos, nCSx, nCSy) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  nCSx=ite(nWRx==nBKx,nWRx,ite(nWRx>nBKx,nWRx-1,nWRx+1));
  nCSy=ite(nWRy==nBKy,nWRy,ite(nWRy>nBKy,nWRy-1,nWRy+1));
}

/* White king (after his moves) is closer to the critical square in sence of Manhattan distance: */

procedure ApproachCriticalSquare(nPos1, nPos2, bApproachCriticalSquare) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call CriticalSquare(nPos1, nCSx1, nCSy1);
  call CriticalSquare(nPos2, nCSx2, nCSy2);
  call ManhattanDistance(nWKx1, nWKy1, nCSx1, nCSy1, nMD1);
  call ManhattanDistance(nWKx2, nWKy2, nCSx2, nCSy2, nMD2);
  bApproachCriticalSquare = nMD1>nMD2;
}

/* Pieces forming the so-called L-shaped pattern: */

procedure LPattern(nPos, bLPattern) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bLPattern =
  (nWKy==nBKy && (nWKx==nBKx+2 || nBKx==nWKx+2) && nWRx==nWKx && (nWRy==nWKy+1 || nWKy==nWRy+1))
  ||
  (nWKx==nBKx && (nWKy==nBKy+2 || nBKy==nWKy+2) && nWRy==nWKy && (nWRx==nWKx+1 || nWKx==nWRx+1));
}

/* Room is greater than 3: */

procedure RoomGt3(nPos, bRoomGt3) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call Room(nPos, nRoom);
  bRoomGt3 = nRoom > 3;
}

/* White king is on the edge of the chessboard: */

procedure WhiteKingEdge(nPos, bWhiteKingEdge) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bWhiteKingEdge = nWKx==0 || nWKx==7 || nWKy==0 || nWKy==7;
}

/* Distance of the white king and white rook has not decreased after move in sense of Chebyshev distance: */

procedure WhiteKingAndRookNotDiverging(nPos1, nPos2, bWhiteKingAndRookNotDiverging) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call ChebyshevDistance(nWKx1, nWKy1, nWRx1, nWRy1, nCDWKWR1);
  call ChebyshevDistance(nWKx2, nWKy2, nWRx2, nWRy2, nCDWKWR2);
  bWhiteKingAndRookNotDiverging = nCDWKWR1>=nCDWKWR2;
}

/* Diagonal and non-diagonal moves of white king: */

procedure KingDiag(nPos1, nPos2, bKingDiag) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bKingDiag = nWKx2!=nWKx1 && nWKy2!=nWKy1;
}

procedure KingNotDiag(nPos1, nPos2, bKingNotDiag) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bKingNotDiag = nWKx2==nWKx1 || nWKy2==nWKy1;
}

/* Black king and white rook are not placed next to each other: */

procedure NotBlackKingNextRook(nPos, bNotBlackKingNextRook) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bNotBlackKingNextRook = nWRx>nBKx+1 || nBKx>nWRx+1 || nWRy>nBKy+1 || nBKy>nWRy+1;
}

/* White king and white rook are placed next to each other: */

procedure WhiteKingNextRook(nPos, bWhiteKingNextRook) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  bWhiteKingNextRook = !(nWRx>nWKx+1 || nWKx>nWRx+1 || nWRy>nWKy+1 || nWKy>nWRy+1);
}

/* Attempt to divide kings: */

procedure DivideAttempt(nPos1, nPos2, bDivideAttempt) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  bDivideAttempt = ((nBKx1<nWKx1 && nWRx1!=nWKx1-1 && nWRx2==nWKx1-1) || (nBKx1>nWKx1 && nWRx1!=nWKx1+1 && nWRx2==nWKx1+1) || (nBKy1<nWKy1 && nWRy1!=nWKy1-1 && nWRy2==nWKy1-1) || (nBKy1>nWKy1 && nWRy1!=nWKy1+1 && nWRy2==nWKy1+1) || (nWRx1==nWKx1 && nWKx1==nBKx1 && (nWRx2==nWKx1+1 || nWRx2==nWKx1-1)) || (nWRy1==nWKy1 && nWKy1==nBKy1 && (nWRy2==nWKy1+1 || nWRy2==nWKy1-1)));
}

/* ******************************************************************** */
/* Strategy (for White)                                                 */
/* ******************************************************************** */

procedure ImmediateMateCond(nPos1, nPos2, bImmediateMateCond) {
  call LegalMoveWhiteRook(nPos1, nPos2, bLegalMoveWhiteRook);
  call Mate(nPos2, bMate);
  bImmediateMateCond = bLegalMoveWhiteRook && bMate;
}

procedure ReadyToMateCond(nPos1, nPos2, bReadyToMateCond) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call LegalMoveWhite(nPos1, nPos2, bLegalMoveWhite);
  bReadyToMateCond = bLegalMoveWhite &&
  ((nWKx2==2 && nBKx2==0 && nBKy2==1 && nWRx2>1 && nWRy2==2 && (nWKy2==1 || nWKy2==0))
  ||
   (nWKx2==2 && nBKx2==0 && nBKy2==6 && nWRx2>1 && nWRy2==5 && (nWKy2==6 || nWKy2==7))
  ||
   (nWKx2==5 && nBKx2==7 && nBKy2==1 && nWRx2<6 && nWRy2==2 && (nWKy2==1 || nWKy2==0))
  ||
   (nWKx2==5 && nBKx2==7 && nBKy2==6 && nWRx2<6 && nWRy2==5 && (nWKy2==6 || nWKy2==7))
  ||
   (nWKy2==2 && nBKy2==0 && nBKx2==1 && nWRy2>1 && nWRx2==2 && (nWKx2==1 || nWKx2==0))
  ||
   (nWKy2==2 && nBKy2==0 && nBKx2==6 && nWRy2>1 && nWRx2==5 && (nWKx2==6 || nWKx2==7))
  ||
   (nWKy2==5 && nBKy2==7 && nBKx2==1 && nWRy2<6 && nWRx2==2 && (nWKx2==1 || nWKx2==0))
  ||
   (nWKy2==5 && nBKy2==7 && nBKx2==6 && nWRy2<6 && nWRx2==5 && (nWKx2==6 || nWKx2==7)));
}

procedure SqueezeCond(nPos1,nPos2,b)  {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1,bWhiteOnTurn1);
  nWKx2 = nWKx1;
  nWKy2 = nWKy1;
  nBKx2 = nBKx1;
  nBKy2 = nBKy1;
  bWhiteOnTurn2 = !bWhiteOnTurn1;

  nRoom2Min=1000;
  nPos2b=0;
  for(nI=0;nI<=15;nI++) {
      nWRx2 = ite(nI<=7,nI,nWRx1);
      nWRy2 = ite(nI>7,nI-8,nWRy1);

      call Cartesian2Pos(nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2,bWhiteOnTurn2, nPos2n);
      call LegalMoveWhiteRook(nPos1, nPos2n, bLegalMoveWhiteRook);
      call NotWhiteRookExposed(nPos2n, bNotWhiteRookExposed);
      call WhiteRookDivides(nPos2n, bWhiteRookDivides);
      call Stalemate(nPos2n, bStalemate);
      call NewRoomSmaller(nPos1, nPos2n, bNewRoomSmaller);
      call Room(nPos2n, nRoom2);
      bb = (bLegalMoveWhiteRook && bNotWhiteRookExposed && bWhiteRookDivides && !bStalemate && bNewRoomSmaller && nRoom2<nRoom2Min);

      b &&= (!bb || nPos2b!=0 || (nPos2b==nPos2n));
      nPos2b = ite(bb,nPos2n,nPos2b);
      nRoom2Min = ite(bb,nRoom2,nRoom2Min);
  }
  b = (nPos2b!=0) && (nPos2b == nPos2);
}

procedure ApproachDiagCond(nPos1, nPos2, bApproachDiagCond) {
  call LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing);
  call KingDiag(nPos1, nPos2, bKingDiag);
  call ApproachCriticalSquare(nPos1, nPos2, bApproachCriticalSquare);
  call NotWhiteRookExposed(nPos2, bNotWhiteRookExposed);
  call WhiteRookDivides(nPos2, bWhiteRookDivides);
  call LPattern(nPos2, bLPattern);
  call RoomGt3(nPos2, bRoomGt3);
  call WhiteKingEdge(nPos2, bWhiteKingEdge);
  call Stalemate(nPos2, bStalemate);
  bApproachDiagCond = bLegalMoveWhiteKing && bKingDiag && bApproachCriticalSquare && bNotWhiteRookExposed && (bWhiteRookDivides || bLPattern) && (bRoomGt3 || !bWhiteKingEdge) && !bStalemate;
}

procedure ApproachNotDiagCond(nPos1, nPos2, bApproachNotDiagCond) {
  call LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing);
  call KingNotDiag(nPos1, nPos2, bKingNotDiag);
  call ApproachCriticalSquare(nPos1, nPos2, bApproachCriticalSquare);
  call NotWhiteRookExposed(nPos2, bNotWhiteRookExposed);
  call WhiteRookDivides(nPos2, bWhiteRookDivides);
  call LPattern(nPos2, bLPattern);
  call RoomGt3(nPos2, bRoomGt3);
  call WhiteKingEdge(nPos2, bWhiteKingEdge);
  call Stalemate(nPos2, bStalemate);
  bApproachNotDiagCond = bLegalMoveWhiteKing && bKingNotDiag && bApproachCriticalSquare && bNotWhiteRookExposed && (bWhiteRookDivides || bLPattern) && (bRoomGt3 || !bWhiteKingEdge) && !bStalemate;
}

procedure KeepRoomDiagCond(nPos1, nPos2, bKeepRoomDiagCond) {
  call LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing);
  call KingDiag(nPos1, nPos2, bKingDiag);
  call NotWhiteRookExposed(nPos2, bNotWhiteRookExposed);
  call WhiteRookDivides(nPos2, bWhiteRookDivides);
  call WhiteKingAndRookNotDiverging(nPos1, nPos2, bWhiteKingAndRookNotDiverging);
  call RoomGt3(nPos2, bRoomGt3);
  call WhiteKingEdge(nPos2, bWhiteKingEdge);
  call Stalemate(nPos2, bStalemate);
  bKeepRoomDiagCond = bLegalMoveWhiteKing && bKingDiag && bNotWhiteRookExposed && bWhiteRookDivides && bWhiteKingAndRookNotDiverging && (bRoomGt3 || !bWhiteKingEdge) && !bStalemate;
}

procedure KeepRoomNotDiagCond(nPos1, nPos2, bKeepRoomNotDiagCond) {
  call LegalMoveWhiteKing(nPos1, nPos2, bLegalMoveWhiteKing);
  call KingNotDiag(nPos1, nPos2, bKingNotDiag);
  call NotWhiteRookExposed(nPos2, bNotWhiteRookExposed);
  call WhiteRookDivides(nPos2, bWhiteRookDivides);
  call WhiteKingAndRookNotDiverging(nPos1, nPos2, bWhiteKingAndRookNotDiverging);
  call RoomGt3(nPos2, bRoomGt3);
  call WhiteKingEdge(nPos2, bWhiteKingEdge);
  call Stalemate(nPos2, bStalemate);
  bKeepRoomNotDiagCond = bLegalMoveWhiteKing && bKingNotDiag && bNotWhiteRookExposed && bWhiteRookDivides && bWhiteKingAndRookNotDiverging && (bRoomGt3 || !bWhiteKingEdge) && !bStalemate;
}

procedure RookHomeCond(nPos1, nPos2, bRookHomeCond) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call LegalMoveWhiteRook(nPos1, nPos2, bLegalMoveWhiteRook);
  call DivideAttempt(nPos1, nPos2, bDivideAttempt);
  call NotBlackKingNextRook(nPos2, bNotBlackKingNextRook);
  call WhiteKingNextRook(nPos2, bWhiteKingNextRook);
  call Stalemate(nPos2, bStalemate);
  bRookHomeCond = bLegalMoveWhiteRook && bDivideAttempt && (bNotBlackKingNextRook || bWhiteKingNextRook) && !bStalemate;
}

procedure RookSafeCond(nPos1, nPos2, bRookSafeCond) {
  call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);
  call Pos2Cartesian(nPos2, nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2);
  call LegalMoveWhiteRook(nPos1, nPos2, bLegalMoveWhiteRook);
  call ChebyshevDistance(nBKx2, nBKy2, nWRx2, nWRy2, nCDBKWR2);
  call NotBlackKingNextRook(nPos2, bNotBlackKingNextRook);
  call WhiteKingNextRook(nPos2, bWhiteKingNextRook);
  call Stalemate(nPos2, bStalemate);
  bRookSafeCond = bLegalMoveWhiteRook && ((nWRx1!=0 && nWRx2==0) || (nWRx1!=7 && nWRx2==7) || (nWRy1!=0 && nWRy2==0) || (nWRy1!=7 && nWRy2==7)) && (nCDBKWR2>2 || (!bNotBlackKingNextRook && bWhiteKingNextRook)) && !bStalemate;
}

procedure Measure(nPos, nMeasure) {
  call Pos2Cartesian(nPos, nWKx, nWKy, nBKx, nBKy, nWRx, nWRy, bWhiteOnTurn);
  call Room(nPos, nRoom);
  call CriticalSquare(nPos, nCSx, nCSy);
  call ManhattanDistance(nWKx, nWKy, nCSx, nCSy, nMD);
  call NotWhiteRookExposed(nPos, bNotWhiteRookExposed);
  call ThreatenedByStalemate(nPos, bThreatenedByStalemate);
  nMeasure=ite(bNotWhiteRookExposed && !bThreatenedByStalemate,6*nRoom+nMD,100);
}

procedure NextMove(nPos1,nI,nPos2,b)
{
   b1 = (nI>0) && (nI<=24);

   nPos2n=nPos1;
   call Pos2Cartesian(nPos1, nWKx1, nWKy1, nBKx1, nBKy1, nWRx1, nWRy1, bWhiteOnTurn1);

   nWKx2 = nWKx1; 
   nWKx2 = ite((nI==1 || nI==4 || nI==6) && nWKx1>0, nWKx1-1, nWKx2);
   nWKx2 = ite((nI==3 || nI==5 || nI==7) && nWKx1<7, nWKx1+1, nWKx2);
   nWKy2 = nWKy1;
   nWKy2 = ite((nI==1 || nI==2 || nI==3) && nWKy1<7, nWKy1+1, nWKy2);
   nWKy2 = ite((nI==6 || nI==7 || nI==8) && nWKy1>0, nWKy1-1, nWKy2);

   nWRx2 = nWRx1;
   nWRy2 = nWRy1;

   nWRx2 = ite(nI>=9 && nI<=16 && nWRx1!=nI-9, nI-9, nWRx2);
   nWRy2 = ite(nI>=17 && nI<=24 && nWRy1!=nI-17, nI-17, nWRy2);

   nBKx2 = nBKx1;
   nBKy2 = nBKy1;
   bWhiteOnTurn2 = !bWhiteOnTurn1;

   b2=false;
   b2 ||= (nI==1 && nWKx1>0 && nWKy1<7);
   b2 ||= (nI==2 && nWKy1<7);
   b2 ||= (nI==3 && nWKx1<7 && nWKy1<7);
   b2 ||= (nI==4 && nWKx1>0);
   b2 ||= (nI==5 && nWKx1<7);
   b2 ||= (nI==6 && nWKx1>0 && nWKy1>0);
   b2 ||= (nI==7 && nWKy1>0);
   b2 ||= (nI==8 && nWKx1<7 && nWKy1>0);

   b2 ||= (nI>=9 && nI<=16 && nWRx1!=nI-9);
   b2 ||= (nI>=17 && nI<=24 && nWRy1!=nI-17);

   b3 = bWhiteOnTurn1;

   call Cartesian2Pos(nWKx2, nWKy2, nBKx2, nBKy2, nWRx2, nWRy2, bWhiteOnTurn2, nPos2n);    
   nPos2=nPos2n;

   b = b1 && b2 && b3 && (nPos2n==nPos2);
}    

procedure StrategyStep(nPos1,nPos2,b,nStep)
{ 
  b = true; 
  nStep0=0;

  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ImmediateMateCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 1, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ReadyToMateCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 2, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call SqueezeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 3, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ApproachDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 4, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ApproachNotDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 5, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call KeepRoomDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 6, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call KeepRoomNotDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 7, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call RookHomeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 8, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call RookSafeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 9, nStep0);
  }
  b &&= (nStep==nStep0 && nStep!=0);
}

procedure Strategy(nPos1,nPos2,b)
{
  b = true;
  nPos0=np;
  nStep0=0;
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ImmediateMateCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 1, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ReadyToMateCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 2, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call SqueezeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 3, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ApproachDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 4, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call ApproachNotDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 5, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call KeepRoomDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 6, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call KeepRoomNotDiagCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 7, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call RookHomeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 8, nStep0);
  }
  for(nMove=1;nMove<=24;nMove++) {
    call NextMove(nPos1,nMove,nPos0,b1);
    call RookSafeCond(nPos1, nPos0, b2);
    b &&= (!b1 || !b2 || nStep0!=0 || (nPos2==nPos0));
    nStep0 = ite(b1 && b2 && nStep0==0, 9, nStep0);
  }
}

nsImmediateMate  =1;
nsReadyToMate    =2;
nsSqueeze        =3;
nsApproachDiag   =4;
nsApproachNotDiag=5;
nsKeepRoomDiag   =6;
nsKeepRoomNotDiag=7;
nsRookHome       =8;
nsRookSafe       =9;

/* ------------------------------------------------------------------------------- */

/* ******************************************************************** */
/* Lemmas                                                               */
/* ******************************************************************** */

/*Starting from any legal KRK position, after a legal step by the black, the obtained position is again a legal KRK position.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call LegalKRKPosition(nPos3,bLegalKRKPosition3);
bKRKLegalityNotPreserved = (bLegalKRKPosition1 && b1 && b2 && !bLegalKRKPosition3);
assert(bKRKLegalityNotPreserved);
*/
/*RESULT::

No solutions found
[Formula generation: 15.796s; conversion to CNF: 0.444s; total: 16.24s]
[Solving time: 112s]
[Formula size: 73627 variables, 262063 clauses]*/

/*Starting from any legal KRK position, the measure decreases each three moves if:
 - within these three moves the white didn’t used the strategy step ReadyToMate;
 - within these three moves the white didn’t used the steps RookHome and RookSafe;
 - the measure is greater than 24 in the first position in this sequence.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call StrategyStep(nPos3,nPos4,b3,nStep3);
call LegalMoveBlack(nPos4,nPos5,b4);
call StrategyStep(nPos5,nPos6,b5,nStep5);
call LegalMoveBlack(nPos6,nPos7,b6);
call Measure(nPos1,nMeasure1);
call Measure(nPos7,nMeasure7);
bTerminationLemma1 = (bLegalKRKPosition1 && b1 && b2 && b3 && b4 && b5 && b6 &&
nStep1!=nsRookSafe && nStep3!=nsRookSafe && nStep5!=nsRookSafe &&
nStep1!=nsRookHome && nStep3!=nsRookHome && nStep5!=nsRookHome &&
nStep1!=nsReadyToMate && nStep3!=nsReadyToMate && nStep5!=nsReadyToMate &&
nMeasure1>24 && nMeasure1<=nMeasure7);
assert(bTerminationLemma1);
*/
/*RESULT::
No solutions found
[Formula generation: 46.788s; conversion to CNF: 1.676s; total: 48.464s]
[Solving time: 8804.16s]
[Formula size: 227270 variables, 808644 clauses]*/

/*If the white uses the strategy step ReadyToMate, then mate follows after the next move.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call StrategyStep(nPos3,nPos4,b3,nStep3);
call Mate(nPos4,bMate);
bMateAfterReadyToMate = (bLegalKRKPosition1 && b1 && b2 && b3 && nStep1==nsReadyToMate && !bMate);
assert(bMateAfterReadyToMate);
*/
/*RESULT::

No solutions found
[Formula generation: 31.512s; conversion to CNF: 1.132s; total: 32.644s]
[Solving time: 9.112s]
[Formula size: 151228 variables, 537368 clauses]*/

/*Starting from any legal KRK position, the steps RookHome and RookSafe can be used by the white only within the first three moves.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call StrategyStep(nPos3,nPos4,b3,nStep3);
call LegalMoveBlack(nPos4,nPos5,b4);
call StrategyStep(nPos5,nPos6,b5,nStep5);
call LegalMoveBlack(nPos6,nPos7,b6);
call StrategyStep(nPos7,nPos8,b7,nStep7);
bTerminationLemma2 = bLegalKRKPosition1 && b1 && b2 && b3 && b4 && b5 && b6 && b7 && (nStep7==nsRookHome || nStep7==nsRookSafe);
assert_all(bTerminationLemma2);
*/
/*RESULT::

No solutions found
[Formula generation: 61.052s; conversion to CNF: 1.848s; total: 62.9s]
[Solving time: 11823.9s]
[Formula size: 293524 variables, 1045159 clauses]*/

/*If the measure is less or equal to 24 in a position with the white to move, then the game ends in 7 or less moves.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call StrategyStep(nPos3,nPos4,b3,nStep3);
call LegalMoveBlack(nPos4,nPos5,b4);
call StrategyStep(nPos5,nPos6,b5,nStep5);
call LegalMoveBlack(nPos6,nPos7,b6);
call StrategyStep(nPos7,nPos8,b7,nStep7);
call LegalMoveBlack(nPos8,nPos9,b8);
call StrategyStep(nPos9,nPos10,b9,nStep9);
call LegalMoveBlack(nPos10,nPos11,b10);
call StrategyStep(nPos11,nPos12,b11,nStep11);
call LegalMoveBlack(nPos12,nPos13,b12);
call StrategyStep(nPos13,nPos14,b13,nStep13);
call LegalMoveBlack(nPos14,nPos15,b14);
call Measure(nPos1,nMeasure1);
bTerminationLemma3 = (bLegalKRKPosition1 && nMeasure1<=24 && b1 && b2 && b3 && b4 && b5 && b6 && b7 && b8 && b9 && b10 && b11 && b12 && b13 && b14);
assert_all(bTerminationLemma3);
*/
/*RESULT::

No solutions found
[Formula generation: 109.176s; conversion to CNF: 3.428s; total: 112.604s]
[Solving time: 103.012s]
[Formula size: 514701 variables, 1832567 clauses]*/

/*The black cannot reach draw.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
call LegalMoveBlack(nPos2,nPos3,b2);
call Draw(nPos3,bDraw);
bLemmaPartialCorrectness1 = (bLegalKRKPosition1 && b1 && b2 && bDraw);
assert(bLemmaPartialCorrectness1);
*/
/* RESULT::

No solutions found
[Formula generation: 16.08s; conversion to CNF: 0.544s; total: 16.624s]
[Solving time: 1.98s]
[Formula size: 73594 variables, 262305 clauses]*/


/*If an KRK game ends, then the last move was made by the white (i.e., in any legal KRK position with the white on turn, there is a move that the white can play according to the strategy).*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,b1,nStep1);
bLemmaPartialCorrectness2 = (bLegalKRKPosition1 && b1 && !(nStep1>=1 && nStep1<=9));
assert(bLemmaPartialCorrectness2);
*/
/* RESULT::

No solutions found
[Formula generation: 15.64s; conversion to CNF: 0.432s; total: 16.072s]
[Solving time: 0.932s]
[Formula size: 73166 variables, 260601 clauses]*/

/*In any legal KRK position, after the move of the white the reached position is not stalemate.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,bStrategyUsed,nStep1);
call Stalemate(nPos2,bStalemate);
bLemmaPartialCorrectness3 = (bLegalKRKPosition1 && bStrategyUsed && bStalemate);
assert(bLemmaPartialCorrectness3);
*/
/* RESULT::

No solutions found
[Formula generation: 15.764s; conversion to CNF: 0.552s; total: 16.316s]
[Solving time: 21.328s]
[Formula size: 77765 variables, 275804 clauses]*/

/*If, after the move of the white, the black cannot move, then the black is mated.*/
/*
call LegalKRKPosition(nPos1,bLegalKRKPosition1);
call StrategyStep(nPos1,nPos2,bStrategyUsed,nStep1);
call BlackKingCannotMove(nPos2,bBlackKingCannotMove);
call Mate(nPos2,bMate);
bLemmaPartialCorrectness4 = (bLegalKRKPosition1 && bStrategyUsed && bBlackKingCannotMove && !bMate);
assert(bLemmaPartialCorrectness4);
*/
/* RESULT::

No solutions found
[Formula generation: 15.8s; conversion to CNF: 0.544s; total: 16.344s]
[Solving time: 18.068s]
[Formula size: 77765 variables, 275813 clauses]*/

